@@ -506,9 +506,12 @@ namespace zmq
         #endif
 
 #ifdef ZMQ_HAS_RVALUE_REFS
-        inline socket_t(socket_t&& rhs) ZMQ_NOTHROW : ptr(rhs.ptr)
+        inline socket_t(socket_t&& rhs) ZMQ_NOTHROW :
+            ptr(rhs.ptr),
+            ctxptr(rhs.ctxptr)
         {
-            rhs.ptr = NULL;
+            rhs.ptr    = NULL;
+            rhs.ctxptr = NULL;
         }
         inline socket_t& operator=(socket_t&& rhs) ZMQ_NOTHROW
         {
@@ -631,6 +634,8 @@ namespace zmq
                 return (size_t) nbytes;
             if (zmq_errno () == EAGAIN)
                 return 0;
+            if (zmq_errno () == EHOSTUNREACH)
+                return 0;
             throw error_t ();
         }
 
@@ -641,6 +646,8 @@ namespace zmq
                 return true;
             if (zmq_errno () == EAGAIN)
                 return false;
+            if (zmq_errno () == EHOSTUNREACH)
+                return false;
             throw error_t ();
         }
 
@@ -847,7 +854,7 @@ namespace zmq
 
         bool add (zmq::socket_t &socket, short events, std::function<void(void)> &handler)
         {
-            if (0 == zmq_poller_add (poller_ptr, socket.ptr, &handler, events)) {
+            if (0 == zmq_poller_add (poller_ptr, socket.ptr, handler ? &handler : NULL, events)) {
                 poller_events.emplace_back (zmq_poller_event_t ());
                 return true;
             }
@@ -868,7 +875,8 @@ namespace zmq
             int rc = zmq_poller_wait_all (poller_ptr, poller_events.data (), poller_events.size (), static_cast<long>(timeout.count ()));
             if (rc >= 0) {
                 std::for_each (poller_events.begin (), poller_events.begin () + rc, [](zmq_poller_event_t& event) {
-                    (*reinterpret_cast<std::function<void(void)>*> (event.user_data)) ();
+                    if (event.user_data != NULL)
+                        (*reinterpret_cast<std::function<void(void)>*> (event.user_data)) ();
                 });
                 return true;
             }
